package com.topsky.flashlight;

import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.hardware.Camera;
import android.hardware.camera2.CameraAccessException;
import android.hardware.camera2.CameraCharacteristics;
import android.hardware.camera2.CameraManager;
import android.os.Build.VERSION;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** FlashlightPlugin */
public class FlashlightPlugin implements MethodCallHandler {

  private Boolean hasFlashLight;
  private Registrar _registrar;

  /** using with camera1. sdk < 21 */
  private Camera _camera;


  /** using with camera2. sdk >= 21 */
  private CameraManager cameraManager;
  private String cameraId;

  private FlashlightPlugin(Registrar registrar) {
    this._registrar = registrar;

    hasFlashLight = false;

    if (VERSION.SDK_INT < 21) {
      this._camera = this.getCamera();
      hasFlashLight = _registrar.context().getApplicationContext().getPackageManager().hasSystemFeature(PackageManager.FEATURE_CAMERA_FLASH);
    } else {
      Activity activity = _registrar.activity();
      cameraManager = (CameraManager) activity.getSystemService(Context.CAMERA_SERVICE);
      try {
        for (String id : cameraManager.getCameraIdList()) {
          CameraCharacteristics characteristics = cameraManager.getCameraCharacteristics(id);
          Integer facing = characteristics.get(CameraCharacteristics.LENS_FACING);

          if (facing != null && facing.equals(CameraCharacteristics.LENS_FACING_BACK)) {
            cameraId = id;
            hasFlashLight = characteristics.get(CameraCharacteristics.FLASH_INFO_AVAILABLE);
          }
        }
      } catch (CameraAccessException e) {
        e.printStackTrace();
      }
    }

  }

  private Camera getCamera() {
    try {
      return Camera.open();
    } catch (Exception e) {
      System.out.println("Failed to get camera : " + e.toString());
      return null;
    }
  }

  /** Plugin registration. */
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "flashlight");
    channel.setMethodCallHandler(new FlashlightPlugin(registrar));
  }

  @Override
  public void onMethodCall(MethodCall call, Result result) {
    if(call.method.equals("hasFlashLight")) {
      result.success(hasFlashLight);
    } else if(call.method.equals("lightOn")) {
      turnLight(true);
      result.success(null);
    } else if(call.method.equals("lightOff")) {
      turnLight(false);
      result.success(null);
    } else {
      result.notImplemented();
    }
  }


  private void turnLight(boolean on) {

    if (!hasFlashLight) {
      return;
    } else if (VERSION.SDK_INT < 21){
      if (_camera == null) {
        return;
      }

      Camera.Parameters params = _camera.getParameters();
      params.setFlashMode(on ? Camera.Parameters.FLASH_MODE_TORCH : Camera.Parameters.FLASH_MODE_OFF);
      _camera.setParameters(params);
      _camera.startPreview();
    } else {
      cameraManager.setTorchMode(cameraId, on);
    }

  }

}
